@@ -1,4 +1,4 @@
-/* $Id: compat.cc,v 1.2 2003/12/21 22:43:57 atterer Exp $ -*- C++ -*-
+/* $Id: compat.cc,v 1.4 2005/07/09 19:14:46 atterer Exp $ -*- C++ -*-
   __   _
   |_) /|  Copyright (C) 2001-2003  |  richard@
   | \/|  Richard Atterer          |  atterer.net
@@ -14,6 +14,7 @@
 #include <config.h>
 
 #include <compat.hh>
+#include <debug.hh>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -29,12 +30,34 @@
 // No additional code required
 //______________________________________________________________________
 
+#elif WINDOWS
+// Truncate using native Windows API
+int compat_truncate(const char* path, uint64 length) {
+  // TODO error handling: GetLastError(), FormatMessage()
+  HANDLE handle = CreateFile(path, GENERIC_READ | GENERIC_WRITE, 0, 0,
+                             OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
+  if (handle == INVALID_HANDLE_VALUE) return -1;
+
+  LARGE_INTEGER targetFileLen;
+  targetFileLen.QuadPart = length;
+  LARGE_INTEGER newFileLen;
+  if (SetFilePointerEx(handle, targetFileLen, &newFileLen, FILE_BEGIN) == 0
+      || SetEndOfFile(handle) == 0U) {
+    CloseHandle(handle);
+    return -1;
+  }
+  Assert(static_cast<uint64>(newFileLen.QuadPart) == length);
+  if (CloseHandle(handle) == 0) return -1;
+  return 0;
+}
+//______________________________________________________________________
+
 #elif !HAVE_TRUNCATE && HAVE_FTRUNCATE
 // Truncate using POSIX ftruncate()
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <unistd.h>
+#include <unistd-jigdo.h>
 #ifndef O_LARGEFILE
 #  define O_LARGEFILE 0 // Linux: Allow 64-bit file sizes on 32-bit arches
 #endif
@@ -46,24 +69,6 @@ int compat_truncate(const char* path, uint64 length) {
 }
 //______________________________________________________________________
 
-#elif !HAVE_TRUNCATE && WINDOWS
-// Truncate using native Windows API
-int compat_truncate(const char* path, uint64 length) {
-  // TODO error handling: GetLastError(), FormatMessage()
-  HANDLE handle = CreateFile(path, GENERIC_READ | GENERIC_WRITE, 0, 0,
-                             OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
-  if (handle == INVALID_HANDLE_VALUE) return -1;
-  LONG lengthHi = length >> 32;
-  if (SetFilePointer(handle, length, &lengthHi, FILE_BEGIN) == 0xffffffffU
-      || SetEndOfFile(handle) == 0U) {
-    CloseHandle(handle);
-    return -1;
-  }
-  if (CloseHandle(handle) == 0) return -1;
-  return 0;
-}
-//______________________________________________________________________
-
 #else
 #  error "No implementation for truncating files!"
 #endif
